@@ -4,7 +4,7 @@
 
   Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 1997 - 2007 Pekka Riikonen
+  Copyright (C) 1997 - 2009 Pekka Riikonen
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -1572,6 +1572,15 @@ void silc_server_send_dest_command_reply(SilcServer server,
   va_end(ap);
 }
 
+/* Send the heartbeat packet. */
+
+void silc_server_send_heartbeat(SilcServer server,
+				SilcPacketStream sock)
+{
+  silc_server_packet_send(server, sock, SILC_PACKET_HEARTBEAT, 0,
+			  NULL, 0);
+}
+
 /* Routine used to send the connection authentication packet. */
 
 void silc_server_send_connection_auth_request(SilcServer server,
