@@ -274,7 +274,7 @@ SilcBool silc_schedule_internal_schedule_fd(SilcSchedule schedule,
   if (!internal)
     return TRUE;
 
-  SILC_LOG_DEBUG(("Scheduling fd %lu, mask %x", task->fd, event_mask));
+  SILC_LOG_DEBUG(("Scheduling fd %u, mask %x", task->fd, event_mask));
 
   memset(&event, 0, sizeof(event));
   if (event_mask & SILC_TASK_READ)
